@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,21 +36,16 @@
 static void
 hide_caret (TmaidWindow *tmaid)
 {
-  GdkRectangle rc;
-
-  gtk_timeout_remove (timer_id);
+  g_source_remove (timer_id);
   timer_id = 0;
   if (caret)
     {
       caret = FALSE;
-      rc.x = edit_get_align_pos (tmaid,
-                                 tmaid->cursor.x, tmaid->cursor.y, FALSE);
-      rc.y = tmaid->cursor.y;
-      rc.x = (rc.x - tmaid->top.x) * tmaid->font_width;
-      rc.y = (rc.y - tmaid->top.y) * tmaid->font_height;
-      rc.width = tmaid->font_width;
-      rc.height = tmaid->font_height;
-      gtk_widget_draw (tmaid->drawing, &rc);
+      gtk_widget_queue_draw_area (tmaid->drawing,
+        (edit_get_align_pos (tmaid, tmaid->cursor.x, tmaid->cursor.y, FALSE)
+                                        - tmaid->top.x) * tmaid->font_width,
+        (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height,
+                                        tmaid->font_width, tmaid->font_height);
     }
 }
 
@@ -75,29 +70,27 @@ draw_caret (TmaidWindow *tmaid,
             && (tmaid->cursor.x != cursor->x || tmaid->cursor.y != cursor->y))
     {
       /* ja:タイマ再設定 */
-      gtk_timeout_remove (timer_id);
+      g_source_remove (timer_id);
       timer_id = 0;
       /* ja:古いキャレットを消去 */
-      rc.x = edit_get_align_pos (tmaid, cursor->x, cursor->y, FALSE);
-      rc.y = cursor->y;
-      rc.x = (rc.x - tmaid->top.x) * tmaid->font_width;
-      rc.y = (rc.y - tmaid->top.y) * tmaid->font_height;
-      rc.width = tmaid->font_width;
-      rc.height = tmaid->font_height;
-      gtk_widget_draw (tmaid->drawing, &rc);
+      gtk_widget_queue_draw_area (tmaid->drawing,
+                    (edit_get_align_pos (tmaid, cursor->x, cursor->y, FALSE)
+                                        - tmaid->top.x) * tmaid->font_width,
+                    (cursor->y - tmaid->top.y) * tmaid->font_height,
+                                        tmaid->font_width, tmaid->font_height);
     }
   /* ja:新しいキャレットを描画 */
   if (timer_id == 0)
-    caret=TRUE;
+    caret = TRUE;
   x = edit_get_align_pos (tmaid, tmaid->cursor.x, tmaid->cursor.y, FALSE);
   y = tmaid->cursor.y;
   rc.x = (x - tmaid->top.x) * tmaid->font_width;
   rc.y = (y - tmaid->top.y) * tmaid->font_height;
   rc.width = tmaid->font_width;
   rc.height = tmaid->font_height;
-  gtk_widget_draw (tmaid->drawing, &rc);
+  gtk_widget_queue_draw_area (tmaid->drawing, rc.x, rc.y, rc.width, rc.height);
   if (timer_id == 0)
-    timer_id = gtk_timeout_add (500, signal_timeout, NULL);
+    timer_id = g_timeout_add (500, signal_timeout, NULL);
   text = g_strdup_printf (_("L%d,C%d"), y + 1, x + 1);
   gtk_label_set_text (GTK_LABEL (label_cursor), text);
   g_free (text);
@@ -242,28 +235,27 @@ clear_sel (TmaidWindow *tmaid,
            GdkPoint    *start,
            GdkPoint    *end)
 {
-  gint st, ed;
-  GdkRectangle rc;
+  gint st, ed, x, y, width, height;
 
   if (start->y == end->y)
     {
       /* ja:同じ行 */
       st = edit_get_align_pos (tmaid, MIN (start->x, end->x), start->y, FALSE);
       ed = edit_get_align_pos (tmaid, MAX (start->x, end->x), start->y, TRUE);
-      rc.x = (st - tmaid->top.x) * tmaid->font_width;
-      rc.y = (start->y - tmaid->top.y) * tmaid->font_height;
-      rc.width = (ed - st + 1) * tmaid->font_width;
-      rc.height = tmaid->font_height;
+      x = (st - tmaid->top.x) * tmaid->font_width;
+      y = (start->y - tmaid->top.y) * tmaid->font_height;
+      width = (ed - st + 1) * tmaid->font_width;
+      height = tmaid->font_height;
     }
   else
     {
       /* ja:違う行 */
-      rc.x = 0;
-      rc.y = (MIN (start->y, end->y) - tmaid->top.y) * tmaid->font_height;
-      rc.width = tmaid->drawing->allocation.width;
-      rc.height = (ABS (start->y - end->y) + 1) * tmaid->font_height;
+      x = 0;
+      y = (MIN (start->y, end->y) - tmaid->top.y) * tmaid->font_height;
+      width = tmaid->drawing->allocation.width;
+      height = (ABS (start->y - end->y) + 1) * tmaid->font_height;
     }
-  gtk_widget_draw (tmaid->drawing, &rc);
+  gtk_widget_queue_draw_area (tmaid->drawing, x, y, width, height);
 }
 
 
@@ -298,10 +290,8 @@ edit_operation (TmaidWindow    *tmaid,
                 const gboolean  caret,
                 const gboolean  select)
 {
-  gint i, j, sx, sy, max, delete, put;
-  GdkPoint cursor_old, cursor_new, top;
-  GdkRectangle rc;
-  LineBuffer *p, *q;
+  gint sx, sy, max;
+  GdkPoint top;
   TmaidHistory *d;
 
   hide_caret (tmaid);
@@ -320,6 +310,9 @@ edit_operation (TmaidWindow    *tmaid,
   else
     {
       /* ja:削除 */
+      gint i, delete;
+      LineBuffer *p;
+
       d->length = edit_get_sel_bytes (tmaid, &tmaid->cursor, &tmaid->select);
       d->text = g_malloc (d->length * sizeof (gchar));
       edit_cpy_sel_mem (tmaid, &tmaid->cursor, &tmaid->select, d->text);
@@ -340,11 +333,10 @@ edit_operation (TmaidWindow    *tmaid,
         }
       tmaid->select.x = -1;
       /* ja:削除した行の削除した桁より右を初期化(右マージンのために1つ多い) */
-      rc.x = (tmaid->cursor.x - tmaid->top.x - 1) * tmaid->font_width;
-      rc.y = (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height;
-      rc.width = tmaid->drawing->allocation.width - rc.x;
-      rc.height = tmaid->font_height;
-      gtk_widget_draw (tmaid->drawing, &rc);
+      i = (tmaid->cursor.x - tmaid->top.x - 1) * tmaid->font_width;
+      gtk_widget_queue_draw_area (tmaid->drawing,
+                    i, (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height,
+                    tmaid->drawing->allocation.width - i, tmaid->font_height);
       /* ja:減った行数分だけスクロール */
       misc_scroll_window (tmaid->drawing, 0, -delete * tmaid->font_height,
         0,
@@ -358,14 +350,10 @@ edit_operation (TmaidWindow    *tmaid,
                                                         p; i++, p = p->next)
         if (!p->margin)
           break;
-      if (i>0)
-        {
-          rc.x = 0;
-          rc.y = (tmaid->cursor.y - tmaid->top.y + 1) * tmaid->font_height;
-          rc.width = tmaid->drawing->allocation.width;
-          rc.height = i * tmaid->font_height;
-          gtk_widget_draw (tmaid->drawing, &rc);
-        }
+      if (i > 0)
+        gtk_widget_queue_draw_area (tmaid->drawing,
+                0, (tmaid->cursor.y - tmaid->top.y + 1) * tmaid->font_height,
+                    tmaid->drawing->allocation.width, i * tmaid->font_height);
     }
   if (length <= 0 || !text)
     {
@@ -375,6 +363,10 @@ edit_operation (TmaidWindow    *tmaid,
   else
     {
       /* ja:挿入 */
+      gint i, j, put;
+      GdkPoint cursor_old, cursor_new;
+      LineBuffer *p, *q;
+
       tmaid->cursor.x = edit_get_align_pos (tmaid,
                                     tmaid->cursor.x, tmaid->cursor.y, FALSE);
       put = edit_put_mem (tmaid, &tmaid->cursor, &cursor_new, text, length);
@@ -401,11 +393,10 @@ edit_operation (TmaidWindow    *tmaid,
                                                             GDK_CURRENT_TIME);
         }
       /* ja:挿入した行の挿入した桁より右を初期化 */
-      rc.x = (cursor_old.x - tmaid->top.x) * tmaid->font_width;
-      rc.y = (cursor_old.y - tmaid->top.y) * tmaid->font_height;
-      rc.width = tmaid->drawing->allocation.width;
-      rc.height = tmaid->font_height;
-      gtk_widget_draw (tmaid->drawing, &rc);
+      gtk_widget_queue_draw_area (tmaid->drawing,
+                            (cursor_old.x - tmaid->top.x) * tmaid->font_width,
+                            (cursor_old.y - tmaid->top.y) * tmaid->font_height,
+                        tmaid->drawing->allocation.width, tmaid->font_height);
       /* ja:増えた行数分だけスクロール */
       misc_scroll_window (tmaid->drawing, 0, put * tmaid->font_height,
             0,
@@ -421,13 +412,9 @@ edit_operation (TmaidWindow    *tmaid,
       while (q->next && q->margin)
         j++, q = q->next;
       if (j > 0)
-        {
-          rc.x = 0;
-          rc.y = (i - tmaid->top.y + 1) * tmaid->font_height;
-          rc.width = tmaid->drawing->allocation.width;
-          rc.height = j * tmaid->font_height;
-          gtk_widget_draw (tmaid->drawing, &rc);
-        }
+        gtk_widget_queue_draw_area (tmaid->drawing,
+                                0, (i - tmaid->top.y + 1) * tmaid->font_height,
+                    tmaid->drawing->allocation.width, j * tmaid->font_height);
     }
   if (tmaid->cursor.x < tmaid->top.x)
     tmaid->top.x = tmaid->cursor.x;
@@ -768,7 +755,7 @@ margin_operation (TmaidWindow *tmaid)
             MAX (tmaid->drawing->allocation.height / tmaid->font_height, 1),
                                                                 tmaid->top.y);
   set_menu_bar (tmaid);
-  gtk_widget_draw (tmaid->drawing, NULL);
+  gtk_widget_queue_draw (tmaid->drawing);
 }
 
 
@@ -823,5 +810,5 @@ tab_operation (TmaidWindow *tmaid)
                     orz_mdi_get_page_from_data (ORZ_MDI (mdi), tmaid), TRUE);
   tmaid->select.x = -1;
   set_menu_bar (tmaid);
-  gtk_widget_draw (tmaid->drawing, NULL);
+  gtk_widget_queue_draw (tmaid->drawing);
 }
