@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,16 +38,14 @@ signal_timeout (gpointer data)
   page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
   if (page >= 0)
     {
-      GdkRectangle rc;
       TmaidWindow *tmaid;
 
       tmaid = orz_mdi_get_data (ORZ_MDI (mdi), page);
-      rc.x = (edit_get_align_pos (tmaid, tmaid->cursor.x,
-                tmaid->cursor.y, FALSE) - tmaid->top.x) * tmaid->font_width;
-      rc.y = (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height;
-      rc.width = tmaid->font_width;
-      rc.height = tmaid->font_height;
-      gtk_widget_draw (tmaid->drawing, &rc);
+      gtk_widget_queue_draw_area (tmaid->drawing,
+            (edit_get_align_pos (tmaid, tmaid->cursor.x,
+                tmaid->cursor.y, FALSE) - tmaid->top.x) * tmaid->font_width,
+            (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height,
+                                        tmaid->font_width, tmaid->font_height);
     }
   return TRUE;
 }
@@ -214,7 +212,7 @@ signal_destroy_draw (GtkWidget   *widget,
   g_object_unref (tmaid->layout);
   if (tmaid->timer_id != 0)
     {
-      gtk_timeout_remove (tmaid->timer_id);
+      g_source_remove (tmaid->timer_id);
       tmaid->timer_id = 0;
     }
   while (tmaid->start->prev)
@@ -972,7 +970,7 @@ signal_timeout_draw (TmaidWindow *tmaid)
       if (!result)
         {
           /* ja:スクロールが終わったらタイマは削除 */
-          gtk_timeout_remove (tmaid->timer_id);
+          g_source_remove (tmaid->timer_id);
           tmaid->timer_id = 0;
         }
     }
@@ -1002,11 +1000,11 @@ signal_motion_notify (GtkWidget      *widget,
       y = event->y;
       state = event->state;
     }
-  if ((state&GDK_BUTTON1_MASK)
+  if ((state & GDK_BUTTON1_MASK)
                             && signal_motion_notify_draw (tmaid, x, y, state)
                                                     && tmaid->timer_id == 0)
     /* ja:はじめてスクロールしたとき */
-    tmaid->timer_id = gtk_timeout_add (1, (GtkFunction)signal_timeout_draw,
+    tmaid->timer_id = g_timeout_add (1, (GSourceFunc)signal_timeout_draw,
                                                                         tmaid);
   return TRUE;
 }
@@ -1017,9 +1015,9 @@ signal_button_release (GtkWidget      *widget,
                        GdkEventButton *event,
                        TmaidWindow    *tmaid)
 {
-  if (tmaid->timer_id!=0)
+  if (tmaid->timer_id != 0)
     {
-      gtk_timeout_remove (tmaid->timer_id);
+      g_source_remove (tmaid->timer_id);
       tmaid->timer_id = 0;
     }
   return TRUE;
